/*
----------------------------------------------------------------------------
Library: 	KEYPAD32
Version:	1.0.0
Date:		12/04/2016
Author:		BisoTronic
Copyright:	(C) 2014-2015 Seletronica s.a.s.
Copyright:	(C) 2016      BisoTronic 
License: 	Creative Commons license 3.0 - Attribution-ShareAlike
----------------------------------------------------------------------------
Purpose:	This library gives access to Archiduino LCD custom board and its
			user customizable buttons.
			Its use is intended only for Archiduino 32U4 modular controller, 
			made in Italy by Seletronica s.a.s.
----------------------------------------------------------------------------
Websites:	http://www.archiduino.com/
			http://www.seletronica.com 
			http://www.bisotronic.it
----------------------------------------------------------------------------
Licensing:	This library is free software; you can redistribute it and/or
			modify it under the terms of the GNU Lesser General Public
			License as published by the Free Software Foundation; either
			version 2.1 of the License, or (at your option) any later version.
----------------------------------------------------------------------------
Disclaimer:	This library is published "as is", without technical support, 
			and with no warranty, express or implied, as to its use for any 
			purpose. IN NO EVENT Seletronica OR BisoTronic OR ITS LICENSORS BE 
			LIABLE FOR ANY DIRECT, INDIRECT, CONSEQUENTIAL, INCIDENTAL OR 
			SPECIAL DAMAGES, INCLUDING, WITHOUT LIMITATION, LOSS OF REVENUE, 
			LOSS OF PROFITS, LOSS ARISING FROM INTERRUPTION OR LOSS OF DATA, 
			REGARDLESS OF THE FORM OF ACTION OR LEGAL THEORY ON WHICH THE 
			LIABILITY MAY BE EXERCISED, EVEN IF ADVISED OF THE POSSIBILITY 
			OR LIKELIHOOD OF SUCH DAMAGES.
----------------------------------------------------------------------------
*/

#include <Arduino.h>
#include <Archiduino32.h>
#include <KEYPAD32.h>
#include <PINS32.h>

//static const uint8_t LCD_BUTTON  = PB4;

KEYPAD32::KEYPAD32(){
	//led run is output
	pinModeEx(LED_RUN, OUTPUT);
	_keyOff = 1;
}

void KEYPAD32::ledOff() {
	//led run off
	digitalWriteEx(LED_RUN, LOW);  
}

void KEYPAD32::ledOn() {
	//led run on
	digitalWriteEx(LED_RUN, HIGH); 
}

uint8_t KEYPAD32::readButton() {

	_keyValue = analogRead(LCD_BUTTON);
	float _r = _keyValue/(_keyOff * 2.0);
	return (uint8_t)floor((15-30 * _r)/(1 - _r) + 0.5);
}

void KEYPAD32::InitKeypad() {

	_keyOff = analogRead(LCD_BUTTON);
}

bool KEYPAD32::IsButton1Pressed(){
	return (readButton() & 0x01);
}

bool KEYPAD32::IsButton2Pressed(){
	return (readButton() & 0x02);
}

bool KEYPAD32::IsButton3Pressed(){
	return (readButton() & 0x04);
}

bool KEYPAD32::IsButton4Pressed(){
	return (readButton() & 0x08);
}



