/*
----------------------------------------------------------------------------
Library: 	PINS32
Version:	1.0.0
Date:		12/04/2016
Author:		BisoTronic
Copyright:	(C) 2014-2015 Seletronica s.a.s.
Copyright:	(C) 2016      BisoTronic 
License: 	Creative Commons license 3.0 - Attribution-ShareAlike
----------------------------------------------------------------------------
Purpose:	This library gives access to Archiduino pins and ports.
			Its use is intended only for Archiduino 32U4 modular controller, 
			made in Italy by Seletronica s.a.s.
----------------------------------------------------------------------------
Websites:	http://www.archiduino.com/
			http://www.seletronica.com 
			http://www.bisotronic.it
----------------------------------------------------------------------------
Licensing:	This library is free software; you can redistribute it and/or
			modify it under the terms of the GNU Lesser General Public
			License as published by the Free Software Foundation; either
			version 2.1 of the License, or (at your option) any later version.
----------------------------------------------------------------------------
Disclaimer:	This library is published "as is", without technical support, 
			and with no warranty, express or implied, as to its use for any 
			purpose. IN NO EVENT Seletronica OR BisoTronic OR ITS LICENSORS BE 
			LIABLE FOR ANY DIRECT, INDIRECT, CONSEQUENTIAL, INCIDENTAL OR 
			SPECIAL DAMAGES, INCLUDING, WITHOUT LIMITATION, LOSS OF REVENUE, 
			LOSS OF PROFITS, LOSS ARISING FROM INTERRUPTION OR LOSS OF DATA, 
			REGARDLESS OF THE FORM OF ACTION OR LEGAL THEORY ON WHICH THE 
			LIABILITY MAY BE EXERCISED, EVEN IF ADVISED OF THE POSSIBILITY 
			OR LIKELIHOOD OF SUCH DAMAGES.
----------------------------------------------------------------------------
*/
/*

  original file:
  pins_arduino.h - Pin definition functions for Arduino Leonardo
  Part of Arduino - http://www.arduino.cc/

  Modified from the original version of Arduino Leonardo
  and adapted for Archiduino to implement the needed ports

*/

#ifndef PINS32_h
#define PINS32_h

#include <Arduino.h>

#ifndef ARDUINO_MAIN
#define PA 1
#define PB 2
#define PC 3
#define PD 4
#define PE 5
#define PF 6
#define PG 7
#define PH 8
#define PJ 10
#define PK 11
#define PL 12
#endif

#define NOT_CONNECT	32

#ifdef __AVR_ATmega32U4__
// Mapping pin port
#define  PB0 17
#define  PB1 15
#define  PB2 16
#define  PB3 14
#define  PB4 26
#define  PB5 27
#define  PB6 28
#define  PB7 11
#define  PC6 5
#define  PC7 13
#define  PD0 3
#define  PD1 2
#define  PD2 0
#define  PD3 1
#define  PD4 24
#define  PD5 30
#define  PD6 29
#define  PD7 25
#define  PE2 31
#define  PE6 7
#define  PF0 23
#define  PF1 22
#define  PF4 21
#define  PF5 20
#define  PF6 19
#define  PF7 18


// Mapping pin of slot modules
static const uint8_t M1A = PB3;
static const uint8_t M1B = PB2;
static const uint8_t M1C = PB1;
static const uint8_t M1D = PB0;
static const uint8_t M2A = PD3;
static const uint8_t M2B = PD2;
static const uint8_t M2C = PB7;
static const uint8_t M3A = PB0;
static const uint8_t M3B = PB7;
static const uint8_t M3C = NOT_CONNECT;
static const uint8_t M4A = PC7;
static const uint8_t M4B = PD4;
static const uint8_t M4C = PC7;
static const uint8_t M5A = PC6;
static const uint8_t M5B = PD7;
static const uint8_t M5C = PC6;
static const uint8_t M6A = PD0;
static const uint8_t M6B = PD1;
static const uint8_t M6C = PD4;
static const uint8_t M7A = PF4;
static const uint8_t M7B = PF5;
static const uint8_t M7C = NOT_CONNECT;
static const uint8_t M8A = PF6;
static const uint8_t M8B = PF7;
static const uint8_t M8C = NOT_CONNECT;
static const uint8_t M9A = PF0;
static const uint8_t M9B = PF1;
static const uint8_t M9C = NOT_CONNECT;
static const uint8_t M10A = PD7;
static const uint8_t M10B = PD6;
static const uint8_t M10C = PD6;
static const uint8_t M11A = PE6;
static const uint8_t M11B = PB4;
static const uint8_t M11C = PE6;
static const uint8_t M12A = PB5;
static const uint8_t M12B = PB6;
static const uint8_t M12C = PB5;
//implemented output
static const uint8_t LED_RUN = PE2;
static const uint8_t LCD_BKLIGHT = PD5;
static const uint8_t LCD_BUTTON = PB4;
static const uint8_t RELE_4 = PC7;
static const uint8_t RELE_5 = PC6;
static const uint8_t RELE_6 = PD4;
static const uint8_t RELE_10 = PD6;
static const uint8_t RELE_11 = PE6;
static const uint8_t RELE_12 = PB5;

#endif

#ifndef Pins_Arduino_h
// Mapping of analog pins as digital I/O
// A6-A11 share with digital pins
static const uint8_t A0 = 18;
static const uint8_t A1 = 19;
static const uint8_t A2 = 20;
static const uint8_t A3 = 21;
static const uint8_t A4 = 22;
static const uint8_t A5 = 23;
static const uint8_t A6 = 24;
static const uint8_t A7 = 25;
static const uint8_t A8 = 26;
static const uint8_t A9 = 27;
static const uint8_t A10 = 28;
static const uint8_t A11 = 29;
#endif

const uint8_t PROGMEM uP32u4_port_mask[] = {

	PD, // D0 - PD2
	PD,	// D1 - PD3
	PD, // D2 - PD1
	PD,	// D3 - PD0
	PD,	// D4 - PD4
	PC, // D5 - PC6
	PD, // D6 - PD7
	PE, // D7 - PE6
	
	PB, // D8 - PB4
	PB,	// D9 - PB5
	PB, // D10 - PB6
	PB,	// D11 - PB7
	PD, // D12 - PD6
	PC, // D13 - PC7
	
	PB,	// D14 - MISO - PB3
	PB,	// D15 - SCK - PB1
	PB,	// D16 - MOSI - PB2
	PB,	// D17 - SS - PB0
	
	PF,	// D18 - A0 - PF7
	PF, // D19 - A1 - PF6
	PF, // D20 - A2 - PF5
	PF, // D21 - A3 - PF4
	PF, // D22 - A4 - PF1
	PF, // D23 - A5 - PF0
	
	PD, // D24 - A6 - PD4
	PD, // D25 - A7 - PD7
	PB, // D26 - A8 - PB4
	PB, // D27 - A9 - PB5
	PB, // D28 - A10 - PB6
	PD, // D29 - A11 - PD6

	//implemented port
	PD, // D30 - PD5
	PE, // D31 - PE2

	NOT_CONNECT,  //32

};


const uint8_t PROGMEM uP32u4_bit_mask[] = {

	_BV(2), // D0 - PD2
	_BV(3),	// D1 - PD3
	_BV(1), // D2 - PD1
	_BV(0),	// D3 - PD0
	_BV(4),	// D4 - PD4
	_BV(6), // D5 - PC6
	_BV(7), // D6 - PD7
	_BV(6), // D7 - PE6
	
	_BV(4), // D8 - PB4
	_BV(5),	// D9 - PB5
	_BV(6), // D10 - PB6
	_BV(7),	// D11 - PB7
	_BV(6), // D12 - PD6
	_BV(7), // D13 - PC7
	
	_BV(3),	// D14 - MISO - PB3
	_BV(1),	// D15 - SCK - PB1
	_BV(2),	// D16 - MOSI - PB2
	_BV(0),	// D17 - SS - PB0
	
	_BV(7),	// D18 - A0 - PF7
	_BV(6), // D19 - A1 - PF6
	_BV(5), // D20 - A2 - PF5
	_BV(4), // D21 - A3 - PF4
	_BV(1), // D22 - A4 - PF1
	_BV(0), // D23 - A5 - PF0
	
	_BV(4), // D24 - A6 - PD4
	_BV(7), // D25 - A7 - PD7
	_BV(4), // D26 - A8 - PB4
	_BV(5), // D27 - A9 - PB5
	_BV(6), // D28 - A10 - PB6
	_BV(6), // D29 - A11 - PD6

	//implemented bit
	_BV(5), // D30 - PD5
	_BV(2), // D31 - PE2

	NOT_CONNECT,  //32
};


const uint8_t PROGMEM uP32u4_timer_mask[] = {

	NOT_ON_TIMER,	
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	TIMER0B,		// 3 
	NOT_ON_TIMER,
	TIMER3A,		// 5 
	TIMER4D,		// 6 
	NOT_ON_TIMER,	
	
	NOT_ON_TIMER,	
	TIMER1A,		// 9 
	TIMER1B,		// 10 
	TIMER0A,		// 11 
	
	NOT_ON_TIMER,	
	TIMER4A,		// 13 
	
	NOT_ON_TIMER,	
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,

	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,

	//implemented timer
	NOT_ON_TIMER,
	NOT_ON_TIMER,
};


#endif /* PINS32_h */
