/*
----------------------------------------------------------------------------
Library: 	wiring_digital_ex
Version:	1.0.0
Date:		12/04/2016
Author:		BisoTronic
Copyright:	(C) 2014-2015 Seletronica s.a.s.
Copyright:	(C) 2016      BisoTronic 
License: 	Creative Commons license 3.0 - Attribution-ShareAlike
----------------------------------------------------------------------------
Purpose:	This library gives access to Archiduino LCD custom board and its
			user customizable buttons.
			Its use is intended only for Archiduino 32U4 modular controller, 
			made in Italy by Seletronica s.a.s.
----------------------------------------------------------------------------
Websites:	http://www.archiduino.com/
			http://www.seletronica.com 
			http://www.bisotronic.it
----------------------------------------------------------------------------
Licensing:	This library is free software; you can redistribute it and/or
			modify it under the terms of the GNU Lesser General Public
			License as published by the Free Software Foundation; either
			version 2.1 of the License, or (at your option) any later version.
----------------------------------------------------------------------------
Disclaimer:	This library is published "as is", without technical support, 
			and with no warranty, express or implied, as to its use for any 
			purpose. IN NO EVENT Seletronica OR BisoTronic OR ITS LICENSORS BE 
			LIABLE FOR ANY DIRECT, INDIRECT, CONSEQUENTIAL, INCIDENTAL OR 
			SPECIAL DAMAGES, INCLUDING, WITHOUT LIMITATION, LOSS OF REVENUE, 
			LOSS OF PROFITS, LOSS ARISING FROM INTERRUPTION OR LOSS OF DATA, 
			REGARDLESS OF THE FORM OF ACTION OR LEGAL THEORY ON WHICH THE 
			LIABILITY MAY BE EXERCISED, EVEN IF ADVISED OF THE POSSIBILITY 
			OR LIKELIHOOD OF SUCH DAMAGES.
----------------------------------------------------------------------------
*/
/*

  original file:
  wiring_digital.c - digital input and output extended functions
  Part of Arduino - http://www.arduino.cc/

  Modified from the original version of Arduino Leonardo
  and adapted for Archiduino to implement the needed ports

*/

#include <Arduino.h>
#include <PINS32.h>

#ifdef __cplusplus
extern "C"{
#endif

#ifndef digitalPinToPortEx
#define digitalPinToPortEx(P) ( pgm_read_byte( uP32u4_port_mask + (P) ) )
#endif
#ifndef digitalPinToBitMaskEx
#define digitalPinToBitMaskEx(P) ( pgm_read_byte( uP32u4_bit_mask + (P) ) )
#endif
#ifndef digitalPinToTimerEx
#define digitalPinToTimerEx(P) ( pgm_read_byte( uP32u4_timer_mask + (P) ) )
#endif
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif

// Forcing this inline keeps the callers from having to push their own stuff
// on the stack. It is a good performance win and only takes 1 more byte per
// user than calling. (It will take more bytes on the 168.)
//
// But shouldn't this be moved into pinMode? Seems silly to check and do on
// each digitalread or write.
//
// Mark Sproul:
// - Removed inline. Save 170 bytes on atmega1280
// - changed to a switch statment; added 32 bytes but much easier to read and maintain.
// - Added more #ifdefs, now compiles for atmega645
//
//static inline void turnOffPWM(uint8_t timer) __attribute__ ((always_inline));
//static inline void turnOffPWM(uint8_t timer)

static void turnOffPwmEx(uint8_t timer)	// TODO: Da sistemare
{
	switch (timer)
	{
		#if defined(TCCR1A) && defined(COM1A1)
		case TIMER1A:   cbi(TCCR1A, COM1A1);    break;
		#endif
		#if defined(TCCR1A) && defined(COM1B1)
		case TIMER1B:   cbi(TCCR1A, COM1B1);    break;
		#endif
		
		#if defined(TCCR2) && defined(COM21)
		case  TIMER2:   cbi(TCCR2, COM21);      break;
		#endif
		
		#if defined(TCCR0A) && defined(COM0A1)
		case  TIMER0A:  cbi(TCCR0A, COM0A1);    break;
		#endif
		
		#if defined(TIMER0B) && defined(COM0B1)
		case  TIMER0B:  cbi(TCCR0A, COM0B1);    break;
		#endif
		#if defined(TCCR2A) && defined(COM2A1)
		case  TIMER2A:  cbi(TCCR2A, COM2A1);    break;
		#endif
		#if defined(TCCR2A) && defined(COM2B1)
		case  TIMER2B:  cbi(TCCR2A, COM2B1);    break;
		#endif
		
		#if defined(TCCR3A) && defined(COM3A1)
		case  TIMER3A:  cbi(TCCR3A, COM3A1);    break;
		#endif
		#if defined(TCCR3A) && defined(COM3B1)
		case  TIMER3B:  cbi(TCCR3A, COM3B1);    break;
		#endif
		#if defined(TCCR3A) && defined(COM3C1)
		case  TIMER3C:  cbi(TCCR3A, COM3C1);    break;
		#endif

		#if defined(TCCR4A) && defined(COM4A1)
		case  TIMER4A:  cbi(TCCR4A, COM4A1);    break;
		#endif					
		#if defined(TCCR4A) && defined(COM4B1)
		case  TIMER4B:  cbi(TCCR4A, COM4B1);    break;
		#endif
		#if defined(TCCR4A) && defined(COM4C1)
		case  TIMER4C:  cbi(TCCR4A, COM4C1);    break;
		#endif			
		#if defined(TCCR4C) && defined(COM4D1)
		case TIMER4D:	cbi(TCCR4C, COM4D1);	break;
		#endif			
			
		#if defined(TCCR5A)
		case  TIMER5A:  cbi(TCCR5A, COM5A1);    break;
		case  TIMER5B:  cbi(TCCR5A, COM5B1);    break;
		case  TIMER5C:  cbi(TCCR5A, COM5C1);    break;
		#endif
	}
}

void pinModeEx(uint8_t pin, uint8_t mode)
{
	uint8_t bit = digitalPinToBitMaskEx(pin);
	uint8_t port = digitalPinToPortEx(pin);
	volatile uint8_t *reg, *out;

	if (port == NOT_A_PIN) return;
	if (port == NOT_CONNECT) return;

	// JWS: can I let the optimizer do this?
	reg = portModeRegister(port);
	out = portOutputRegister(port);

	if (mode == INPUT) { 
		uint8_t oldSREG = SREG;
                cli();
		*reg &= ~bit;
		*out &= ~bit;
		SREG = oldSREG;
	} else if (mode == INPUT_PULLUP) {
		uint8_t oldSREG = SREG;
                cli();
		*reg &= ~bit;
		*out |= bit;
		SREG = oldSREG;
	} else {
		uint8_t oldSREG = SREG;
                cli();
		*reg |= bit;
		SREG = oldSREG;
	}
}

void digitalWriteEx(uint8_t pin, uint8_t val)
{
	uint8_t timer = digitalPinToTimerEx(pin);
	uint8_t bit = digitalPinToBitMaskEx(pin);
	uint8_t port = digitalPinToPortEx(pin);
	volatile uint8_t *out;

	if (port == NOT_A_PIN) return;
	if (port == NOT_CONNECT) return;

	// If the pin that support PWM output, we need to turn it off
	// before doing a digital write.
	if (timer != NOT_ON_TIMER)
		turnOffPwmEx(timer);

	out = portOutputRegister(port);

	uint8_t oldSREG = SREG;
	cli();

	if (val == LOW) {
		*out &= ~bit;
	} else {
		*out |= bit;
	}

	SREG = oldSREG;
}

int digitalReadEx(uint8_t pin)
{
	uint8_t timer = digitalPinToTimerEx(pin);
	uint8_t bit = digitalPinToBitMaskEx(pin);
	uint8_t port = digitalPinToPortEx(pin);

	if (port == NOT_A_PIN) return LOW;
	if (port == NOT_CONNECT) return LOW;

	// If the pin that support PWM output, we need to turn it off
	// before getting a digital reading.
	if (timer != NOT_ON_TIMER) turnOffPwmEx(timer);

	if (*portInputRegister(port) & bit) return HIGH;
	return LOW;
}

#ifdef __cplusplus
} // extern "C"
#endif